"use strict";
/**
 * 通用工具类
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyToClipboard = exports.throttle = exports.parallel = exports.dateFormat2 = exports.dateFormat = exports.isDev = exports.isDebug = void 0;
const env = process.env.NODE_ENV;
/**
 * 是否调试模式中
 *
 * 打包成组件之后不能用
 *
 * @returns
 */
function isDebug() {
    return env === 'development';
}
exports.isDebug = isDebug;
function isDev() {
    let currentHostname = window.location.hostname;
    // 判断主机名是否是内网地址
    return (currentHostname.startsWith('192.168.') || currentHostname.startsWith('10.') || currentHostname === 'localhost');
}
exports.isDev = isDev;
/**
* 日期格式化。详见博客文章：http://blog.csdn.net/zhangxin09/archive/2011/01/01/6111294.aspx
* e.g: new Date().format("yyyy-MM-dd hh:mm:ss")
*
* @param 	{String} format
* @return	{String}
*/
function dateFormat(format) {
    let $1, o = {
        "M+": this.getMonth() + 1,
        "d+": this.getDate(),
        "h+": this.getHours(),
        "m+": this.getMinutes(),
        "s+": this.getSeconds(),
        // 季度 quarter
        "q+": Math.floor((this.getMonth() + 3) / 3),
        "S": this.getMilliseconds() // 千秒
    };
    if (/(y+)/.test(format)) {
        $1 = RegExp.$1, format = format.replace($1, String(this.getFullYear()).substr(4 - $1));
    }
    let key, value;
    for (key in o) { // 如果没有指定该参数，则子字符串将延续到 stringvar 的最后。
        if (new RegExp("(" + key + ")").test(format)) {
            $1 = RegExp.$1,
                value = String(o[key]),
                value = $1.length == 1 ? value : ("00" + value).substr(value.length),
                format = format.replace($1, value);
        }
    }
    return format;
}
exports.dateFormat = dateFormat;
/**
 * 日期格式化
 * @author meizz
 * @param date  日期，必须为 Date 类型
 * @param fmt   格式模板
 * @returns 格式化后的字符串
 */
function dateFormat2(date, fmt) {
    let o = {
        "M+": date.getMonth() + 1,
        "d+": date.getDate(),
        "h+": date.getHours(),
        "m+": date.getMinutes(),
        "s+": date.getSeconds(),
        "q+": Math.floor((date.getMonth() + 3) / 3),
        "S": date.getMilliseconds() // 毫秒   
    };
    if (/(y+)/.test(fmt))
        fmt = fmt.replace(RegExp.$1, (date.getFullYear() + "").substr(4 - RegExp.$1.length));
    for (var k in o)
        if (new RegExp("(" + k + ")").test(fmt)) {
            let obj = (RegExp.$1.length == 1) ? o[k] : ("00" + o[k]).substr(("" + o[k]).length);
            // @ts-ignore
            fmt = fmt.replace(RegExp.$1, obj);
        }
    return fmt;
}
exports.dateFormat2 = dateFormat2;
/**
 * 并行和串行任务
 *
 * @author https://segmentfault.com/a/1190000013265925
 * @param arr
 * @param finnaly
 */
function parallel(arr, _finally) {
    let fn, index = 0;
    // @ts-ignore
    let statusArr = Array(arr.length).fill().map(() => ({ isActive: false, data: null }));
    let isFinished = function () {
        return statusArr.every((item) => item.isActive === true);
    };
    let resolve = function (index) {
        return function (data) {
            statusArr[index].data = data;
            statusArr[index].isActive = true;
            let isFinish = isFinished();
            if (isFinish) {
                let datas = statusArr.map((item) => item.data);
                _finally(datas);
            }
        };
    };
    // @ts-ignore
    while ((fn = arr.shift())) {
        fn(resolve(index)); // 给 resolve 函数追加参数,可以使用 bind 函数实现,这里使用了柯里化
        index++;
    }
}
exports.parallel = parallel;
/**
 * 函数节流
 *
 * @author https://www.cnblogs.com/moqiutao/p/6875955.html
 * @param fn
 * @param delay
 * @param mustRunDelay
 */
function throttle(fn, delay, mustRunDelay) {
    var timer, t_start;
    return function () {
        var t_curr = +new Date();
        window.clearTimeout(timer);
        if (!t_start)
            t_start = t_curr;
        if (t_curr - t_start >= mustRunDelay) {
            // @ts-ignore
            fn.apply(this, arguments);
            t_start = t_curr;
        }
        else {
            var args = arguments;
            // @ts-ignore
            timer = window.setTimeout(() => fn.apply(this, args), delay);
        }
    };
}
exports.throttle = throttle;
/**
* 复制文字到剪切板
*
* @param {*} text
*/
function copyToClipboard(text) {
    if (navigator.clipboard) {
        // clipboard api 复制
        navigator.clipboard.writeText(text);
    }
    else {
        let textarea = document.createElement('textarea');
        document.body.appendChild(textarea);
        // 隐藏此输入框
        textarea.style.position = 'fixed';
        textarea.style.clip = 'rect(0 0 0 0)';
        textarea.style.top = '10px';
        // 赋值
        textarea.value = text;
        // 选中
        textarea.select();
        // 复制
        document.execCommand('copy', true);
        // 移除输入框
        document.body.removeChild(textarea);
    }
}
exports.copyToClipboard = copyToClipboard;
//# sourceMappingURL=utils.js.map