"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const EmptyContent_1 = __importDefault(require("../EmptyContent"));
exports.default = {
    props: {
        action: { type: String, required: false },
        limitSize: { type: Number, default: 20000 },
        limitFileType: String,
        isImgUpload: Boolean,
        isShowBtn: { type: Boolean, default: true },
        accpectFileType: {
            type: String, default() {
                if (this.isImgUpload)
                    return 'image/*';
            }
        },
        buttonBottom: Boolean,
        radomId: {
            type: Number, default() {
                return Math.round(Math.random() * 1000);
            },
        },
    },
    data() {
        return {
            model: { children: [] },
            imgSrc: EmptyContent_1.default.empty,
            open: false,
            allowAddNode: false,
            // isFolder : false
            fileName: '',
            fileSize: 0,
            progress: 0,
            errMsg: '',
            newlyId: '',
            fileObj: null
        };
    },
    methods: {
        /**
         * 选择文件后触发的事件
         *
         * @param ev
         */
        onUploadInputChange(ev) {
            let fileInput = ev.target;
            if (!fileInput.files || !fileInput.files[0])
                return;
            // this.errStatus = [false, false, false];
            this.onFileGet(fileInput.files);
            let file = fileInput.files[0];
            this.fileObj = file;
            if (this.isImgUpload) {
                let reader = new FileReader();
                reader.onload = (e) => {
                    let imgBase64 = e.target.result; // 得到了图片的 base64 编码
                    // alert(imgBase64Str);
                    this.imgSrc = imgBase64;
                };
                reader.readAsDataURL(file);
            }
        },
        /**
         * 返回 File 对象，在表单混合上传时候有用
         * @returns
         */
        getFileObj() {
            return this.fileObj;
        },
        onDrop(ev) {
            var _a;
            ev.preventDefault(); // 阻止进行拖拽时浏览器的默认行为，即自动打开图片
            if ((_a = ev.dataTransfer) === null || _a === void 0 ? void 0 : _a.files)
                this.onFileGet(ev.dataTransfer.files);
        },
        onFileGet(files) {
            let file = files[0], fileType = file.type;
            this.$fileObj = file;
            this.fileName = file.name;
            this.fileSize = file.size;
        },
        /**
         * 执行上传
         *
         * @param this
         */
        doUpload() {
            // this.$uploadOk_callback({ isOk: true, msg: "ok!", imgUrl: "fdfdf" });
            // return;
            let fd = new FormData();
            if (this.$blob)
                fd.append("file", this.$blob, this.fileName);
            else if (this.$fileObj)
                fd.append("file", this.$fileObj);
            let xhr = new XMLHttpRequest();
            //@ts-ignore
            xhr.onreadystatechange = aj.xhr.requestHandler.delegate(null, this.$uploadOk_callback, "json");
            xhr.open("POST", this.action, true);
            xhr.onprogress = (ev) => {
                let progress = 0, p = ~~((ev.loaded * 1000) / ev.total);
                p = p / 10;
                if (progress !== p)
                    progress = p;
                this.progress = progress;
            };
            xhr.send(fd);
        },
        changeByte: changeByte
    }
};
/**
 * 字节 Byte 转化成 KB，MB，GB
 *
 * @param limit
 */
function changeByte(limit) {
    let size = "";
    if (limit < 0.1 * 1024)
        // 小于 0.1KB，则转化成 B
        size = limit.toFixed(2) + "B";
    else if (limit < 0.1 * 1024 * 1024)
        // 小于 0.1MB，则转化成 KB
        size = (limit / 1024).toFixed(2) + "KB";
    else if (limit < 0.1 * 1024 * 1024 * 1024)
        // 小于 0.1GB，则转化成 MB
        size = (limit / (1024 * 1024)).toFixed(2) + "MB";
    // 其他转化成 GB
    else
        size = (limit / (1024 * 1024 * 1024)).toFixed(2) + "GB";
    let index = size.indexOf("."); // 获取小数点处的索引
    if (size.substr(index + 1, 2) == "00")
        // 获取小数点后两位的值，判断后两位是否为 00，如果是则删除 00
        return size.substring(0, index) + size.substr(index + 3, 2);
    return size;
}
//# sourceMappingURL=FileUploader.js.map